pub enum EntryRef {
Lookup(EntryName),
Resolved(ResolvedPath),
}Expand description
Entry path passed to callbacks that may run before or after inode resolution.
Lookup is used while resolving a parent/name pair before an inode is attached. Resolved is used when the inode table already has an inode for the entry.
Variants
Lookup(EntryName)
Resolved(ResolvedPath)
Implementations