pub enum ThreadCount {
Default,
NumThreads(usize),
}Expand description
Number of fuser event-loop threads used to serve a mount. This configures fuser session threading, not an internal worker pool.
Variants
Default
Use the current machine parallelism as reported by the standard library.
NumThreads(usize)
Use an explicit number of fuser event-loop threads.