pub enum Xattr {
Size(u32),
Data(Vec<u8>),
}Expand description
Represents the return value from the listxattr and getxattr calls, which can be either a
size or contain data, depending on how they are called.
Variants
Size(u32)
Size needed to read the attribute value or name list.
Data(Vec<u8>)
Attribute data or null-terminated attribute name list.