CallbackResult

Struct CallbackResult 

pub struct CallbackResult { /* private fields */ }
Dummy struct returned by the callback in the read() method. Cannot be constructed outside this crate, read() requires you to return it, thus ensuring that you don’t forget to call the callback.