pub struct EntryName { /* private fields */ }Expand description
Entry name resolved relative to a parent directory path.
Implementations
Sourceimpl EntryName
impl EntryName
Sourcepub fn with(self, ino: Inode) -> ResolvedPath
pub fn with(self, ino: Inode) -> ResolvedPath
Attaches an inode to this entry name.
Sourcepub fn new(parent: FolderPath, name: OsString) -> Self
pub fn new(parent: FolderPath, name: OsString) -> Self
Creates an entry name from a parent folder path and child name.
Sourcepub fn full_path(&self) -> PathBuf
pub fn full_path(&self) -> PathBuf
Returns the full path by joining the parent path and entry name.
Sourcepub fn parent_path(&self) -> Arc<PathBuf> ⓘ
pub fn parent_path(&self) -> Arc<PathBuf> ⓘ
Returns the parent directory path.