pub struct InitFlags(/* private fields */);Expand description
Init request/reply flags.
Implementations
Sourceimpl InitFlags
impl InitFlags
Sourcepub const FUSE_ASYNC_READ: InitFlags
pub const FUSE_ASYNC_READ: InitFlags
asynchronous read requests
Sourcepub const FUSE_POSIX_LOCKS: InitFlags
pub const FUSE_POSIX_LOCKS: InitFlags
remote locking for POSIX file locks
Sourcepub const FUSE_FILE_OPS: InitFlags
pub const FUSE_FILE_OPS: InitFlags
kernel sends file handle for fstat, etc…
Sourcepub const FUSE_ATOMIC_O_TRUNC: InitFlags
pub const FUSE_ATOMIC_O_TRUNC: InitFlags
handles the O_TRUNC open flag in the filesystem
Sourcepub const FUSE_EXPORT_SUPPORT: InitFlags
pub const FUSE_EXPORT_SUPPORT: InitFlags
filesystem handles lookups of “.” and “..”
Sourcepub const FUSE_BIG_WRITES: InitFlags
pub const FUSE_BIG_WRITES: InitFlags
filesystem can handle write size larger than 4kB
Sourcepub const FUSE_DONT_MASK: InitFlags
pub const FUSE_DONT_MASK: InitFlags
don’t apply umask to file mode on create operations
Sourcepub const FUSE_SPLICE_WRITE: InitFlags
pub const FUSE_SPLICE_WRITE: InitFlags
kernel supports splice write on the device
Sourcepub const FUSE_SPLICE_MOVE: InitFlags
pub const FUSE_SPLICE_MOVE: InitFlags
kernel supports splice move on the device
Sourcepub const FUSE_SPLICE_READ: InitFlags
pub const FUSE_SPLICE_READ: InitFlags
kernel supports splice read on the device
Sourcepub const FUSE_FLOCK_LOCKS: InitFlags
pub const FUSE_FLOCK_LOCKS: InitFlags
remote locking for BSD style file locks
Sourcepub const FUSE_HAS_IOCTL_DIR: InitFlags
pub const FUSE_HAS_IOCTL_DIR: InitFlags
kernel supports ioctl on directories
Sourcepub const FUSE_AUTO_INVAL_DATA: InitFlags
pub const FUSE_AUTO_INVAL_DATA: InitFlags
automatically invalidate cached pages
Sourcepub const FUSE_DO_READDIRPLUS: InitFlags
pub const FUSE_DO_READDIRPLUS: InitFlags
do READDIRPLUS (READDIR+LOOKUP in one)
Sourcepub const FUSE_READDIRPLUS_AUTO: InitFlags
pub const FUSE_READDIRPLUS_AUTO: InitFlags
adaptive readdirplus
Sourcepub const FUSE_ASYNC_DIO: InitFlags
pub const FUSE_ASYNC_DIO: InitFlags
asynchronous direct I/O submission
Sourcepub const FUSE_WRITEBACK_CACHE: InitFlags
pub const FUSE_WRITEBACK_CACHE: InitFlags
use writeback cache for buffered writes
Sourcepub const FUSE_NO_OPEN_SUPPORT: InitFlags
pub const FUSE_NO_OPEN_SUPPORT: InitFlags
kernel supports zero-message opens
Sourcepub const FUSE_PARALLEL_DIROPS: InitFlags
pub const FUSE_PARALLEL_DIROPS: InitFlags
allow parallel lookups and readdir
Sourcepub const FUSE_HANDLE_KILLPRIV: InitFlags
pub const FUSE_HANDLE_KILLPRIV: InitFlags
fs handles killing suid/sgid/cap on write/chown/trunc
Sourcepub const FUSE_POSIX_ACL: InitFlags
pub const FUSE_POSIX_ACL: InitFlags
filesystem supports posix acls
Sourcepub const FUSE_ABORT_ERROR: InitFlags
pub const FUSE_ABORT_ERROR: InitFlags
reading the device after abort returns ECONNABORTED
Sourcepub const FUSE_MAX_PAGES: InitFlags
pub const FUSE_MAX_PAGES: InitFlags
init_out.max_pages contains the max number of req pages
Sourcepub const FUSE_CACHE_SYMLINKS: InitFlags
pub const FUSE_CACHE_SYMLINKS: InitFlags
cache READLINK responses
Sourcepub const FUSE_NO_OPENDIR_SUPPORT: InitFlags
pub const FUSE_NO_OPENDIR_SUPPORT: InitFlags
kernel supports zero-message opendir
Sourcepub const FUSE_EXPLICIT_INVAL_DATA: InitFlags
pub const FUSE_EXPLICIT_INVAL_DATA: InitFlags
only invalidate cached pages on explicit request
Sourcepub const FUSE_MAP_ALIGNMENT: InitFlags
pub const FUSE_MAP_ALIGNMENT: InitFlags
map_alignment field is valid
Sourcepub const FUSE_SUBMOUNTS: InitFlags
pub const FUSE_SUBMOUNTS: InitFlags
filesystem supports submounts
Sourcepub const FUSE_HANDLE_KILLPRIV_V2: InitFlags
pub const FUSE_HANDLE_KILLPRIV_V2: InitFlags
fs handles killing suid/sgid/cap on write/chown/trunc (v2)
Sourcepub const FUSE_SETXATTR_EXT: InitFlags
pub const FUSE_SETXATTR_EXT: InitFlags
extended setxattr support
Sourcepub const FUSE_INIT_EXT: InitFlags
pub const FUSE_INIT_EXT: InitFlags
extended fuse_init_in request
Sourcepub const FUSE_INIT_RESERVED: InitFlags
pub const FUSE_INIT_RESERVED: InitFlags
reserved, do not use
Sourcepub const FUSE_SECURITY_CTX: InitFlags
pub const FUSE_SECURITY_CTX: InitFlags
add security context to create/mkdir/symlink/mknod
Sourcepub const FUSE_HAS_INODE_DAX: InitFlags
pub const FUSE_HAS_INODE_DAX: InitFlags
filesystem supports per-inode DAX
Sourcepub const FUSE_CREATE_SUPP_GROUP: InitFlags
pub const FUSE_CREATE_SUPP_GROUP: InitFlags
create with supplementary group
Sourcepub const FUSE_HAS_EXPIRE_ONLY: InitFlags
pub const FUSE_HAS_EXPIRE_ONLY: InitFlags
kernel supports expire-only invalidation
Sourcepub const FUSE_DIRECT_IO_ALLOW_MMAP: InitFlags
pub const FUSE_DIRECT_IO_ALLOW_MMAP: InitFlags
allow mmap for direct I/O files
Sourcepub const FUSE_PASSTHROUGH: InitFlags
pub const FUSE_PASSTHROUGH: InitFlags
filesystem wants to use passthrough files
Sourcepub const FUSE_NO_EXPORT_SUPPORT: InitFlags
pub const FUSE_NO_EXPORT_SUPPORT: InitFlags
filesystem does not support export
Sourcepub const FUSE_HAS_RESEND: InitFlags
pub const FUSE_HAS_RESEND: InitFlags
kernel supports resend requests
Sourcepub const FUSE_ALLOW_IDMAP: InitFlags
pub const FUSE_ALLOW_IDMAP: InitFlags
allow idmapped mounts
Sourcepub const FUSE_OVER_IO_URING: InitFlags
pub const FUSE_OVER_IO_URING: InitFlags
kernel supports io_uring for communication
Sourcepub const FUSE_REQUEST_TIMEOUT: InitFlags
pub const FUSE_REQUEST_TIMEOUT: InitFlags
kernel supports request timeout
Sourceimpl InitFlags
impl InitFlags
Sourcepub const fn bits(&self) -> u64
pub const fn bits(&self) -> u64
Get the underlying bits value.
The returned value is exactly the bits set in this flags value.
Sourcepub const fn from_bits(bits: u64) -> Option<InitFlags>
pub const fn from_bits(bits: u64) -> Option<InitFlags>
Convert from a bits value.
This method will return None if any unknown bits are set.
Sourcepub const fn from_bits_truncate(bits: u64) -> InitFlags
pub const fn from_bits_truncate(bits: u64) -> InitFlags
Convert from a bits value, unsetting any unknown bits.
Sourcepub const fn from_bits_retain(bits: u64) -> InitFlags
pub const fn from_bits_retain(bits: u64) -> InitFlags
Convert from a bits value exactly.
Sourcepub fn from_name(name: &str) -> Option<InitFlags>
pub fn from_name(name: &str) -> Option<InitFlags>
Get a flags value with the bits of a flag with the given name set.
This method will return None if name is empty or doesn’t
correspond to any named flag.
Sourcepub const fn intersects(&self, other: InitFlags) -> bool
pub const fn intersects(&self, other: InitFlags) -> bool
Whether any set bits in other are also set in self.
Sourcepub const fn contains(&self, other: InitFlags) -> bool
pub const fn contains(&self, other: InitFlags) -> bool
Whether all set bits in other are also set in self.
Sourcepub fn remove(&mut self, other: InitFlags)
pub fn remove(&mut self, other: InitFlags)
The intersection of self with the complement of other (&!).
This method is not equivalent to self & !other when other has unknown bits set.
remove won’t truncate other, but the ! operator will.
Sourcepub fn toggle(&mut self, other: InitFlags)
pub fn toggle(&mut self, other: InitFlags)
The bitwise exclusive-or (^) of the bits in self and other.
Sourcepub fn set(&mut self, other: InitFlags, value: bool)
pub fn set(&mut self, other: InitFlags, value: bool)
Call insert when value is true or remove when value is false.
Sourcepub const fn intersection(self, other: InitFlags) -> InitFlags
pub const fn intersection(self, other: InitFlags) -> InitFlags
The bitwise and (&) of the bits in self and other.
Sourcepub const fn union(self, other: InitFlags) -> InitFlags
pub const fn union(self, other: InitFlags) -> InitFlags
The bitwise or (|) of the bits in self and other.
Sourcepub const fn difference(self, other: InitFlags) -> InitFlags
pub const fn difference(self, other: InitFlags) -> InitFlags
The intersection of self with the complement of other (&!).
This method is not equivalent to self & !other when other has unknown bits set.
difference won’t truncate other, but the ! operator will.
Sourcepub const fn symmetric_difference(self, other: InitFlags) -> InitFlags
pub const fn symmetric_difference(self, other: InitFlags) -> InitFlags
The bitwise exclusive-or (^) of the bits in self and other.
Sourcepub const fn complement(self) -> InitFlags
pub const fn complement(self) -> InitFlags
The bitwise negation (!) of the bits in self, truncating the result.