pub struct ResolvedPath { /* private fields */ }Expand description
Path resolved from an inode with the current inode number attached.
Implementations
Sourceimpl ResolvedPath
impl ResolvedPath
Sourcepub fn new(parent: Arc<PathBuf>, name: OsString, ino: Inode) -> Self
pub fn new(parent: Arc<PathBuf>, name: OsString, ino: Inode) -> Self
Creates a resolved path from a parent path, entry name, and inode.
Sourcepub fn full_path(&self) -> PathBuf
pub fn full_path(&self) -> PathBuf
Returns the full path by joining the parent path and entry name.
Sourcepub fn parent_path(&self) -> Arc<PathBuf> ⓘ
pub fn parent_path(&self) -> Arc<PathBuf> ⓘ
Returns the parent directory path.
Sourcepub fn entry_name(&self) -> EntryName
pub fn entry_name(&self) -> EntryName
Returns the same path without the inode context.